import 'package:flutter/material.dart';
import 'package:flutter_boost/boost_navigator.dart';
import 'package:flutter_boost/flutter_boost.dart';
/// 路由状态基础类
/// 提供了页面生命周期监听和APP生命周期监听
/// 需要在MaterialApp.navigatorObservers中注册BaseRouteWeigetState.BaseRouteWeigetStateObserver
abstract class BaseRouteWeigetState<T extends StatefulWidget> extends State<T>
    with PageVisibilityObserver {
  static RouteObserver<PageRoute> weigetStateObserver = RouteObserver();

  bool _isResumed = false;
  bool _isDispose = false;

  @override
  void setState(VoidCallback fn) {
    if (_isDispose) {
      super.setState(fn);
    }
  }

  @override
  void initState() {
    super.initState();
  }

  @override
  void dispose() {
    _isDispose = true;
    PageVisibilityBinding.instance.removeObserver(this);
    super.dispose();
  }

  @override
  void didChangeDependencies() {
    PageRoute route = ModalRoute.of(context) as PageRoute;
    super.didChangeDependencies();

    ///注册监听器
    PageVisibilityBinding.instance.addObserver(this, route);
  }

  @override
  void onBackground() {
    super.onBackground();
    onAppPause();
  }

  @override
  void onForeground() {
    super.onForeground();
    onAppResume();
  }

  @override
  void onPageHide() {
    super.onPageHide();
    _onPause();
  }

  @override
  void onPageShow() {
    super.onPageShow();
    _onResume();
  }

  void _onResume() {
    if (_isResumed) {
      return;
    }
    ModalRoute route = ModalRoute.of(context);
    if (route == null) {
      return;
    }
    RouteSettings settings = route.settings;
    if (settings.name == null) {
      return;
    }
    onResume();
  }

  void _onPause() {
    if (!_isResumed) {
      return;
    }
    ModalRoute route = ModalRoute.of(context);
    if (route == null) {
      return;
    }
    RouteSettings settings = route.settings;
    if (settings.name == null) {
      return;
    }

    onPause();
  }

  /// 页面可见
  void onResume() {
    _isResumed = true;
  }

  /// 页面不可见
  void onPause() {
    _isResumed = false;
  }

  /// 页面是否处于可见状态
  bool isResumed() {
    return _isResumed;
  }

  /// APP可见
  void onAppResume() {}

  /// APP不可见
  void onAppPause() {}

  /// 获取路由名称
  String getRouteName() {
    PageInfo info = BoostNavigator.instance.getTopByContext(context);
    return info.pageName;
  }

  /// 获取路由参数
  T getRouteArgs<T>() {
    PageInfo info = BoostNavigator.instance.getTopByContext(context);
    return info.arguments as T;
  }
}
