import 'package:flutter/material.dart';
import 'base_route_widget_state.dart';
import './base_page_view_item_state.dart';
/// PageView 子项抽象状态
/// 提供了生命周期监听
/// 需要在MaterialApp.navigatorObservers中注册BaseRouteState.baseRouteStateObserver
abstract class BaseTabBarViewItemState<T extends StatefulWidget>
    extends BaseRouteWeigetState<T> with AutomaticKeepAliveClientMixin {
  TabController _oldController;
  @override
  bool get wantKeepAlive => true;

  @override
  void initState() {
    super.initState();
    if (wantKeepAlive) {
      Future.delayed(Duration(milliseconds: 300)).then((value) {
        _controllerListener();
      });
    }
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    _addListener();
  }

  @override
  void didUpdateWidget(covariant T oldWidget) {
    super.didUpdateWidget(oldWidget);
    _addListener();
  }

  void _addListener() {
    if (wantKeepAlive && _oldController != getTabController()) {
      _oldController = getTabController();
      getTabController().addListener(_controllerListener);
    }
    _addParentListener();
  }

  void _addParentListener() {
    BasePageViewItemState pstate =
        context.findAncestorStateOfType<BasePageViewItemState>();
    if (pstate != null && pstate.getPageController() != null) {
      pstate.getPageController().addListener(() {
        getTabController().notifyListeners();
      });
    }
    BaseTabBarViewItemState tstate =
        context.findAncestorStateOfType<BaseTabBarViewItemState>();
    if (tstate != null && tstate.getTabController() != null) {
      tstate.getTabController().addListener(() {
        getTabController().notifyListeners();
      });
    }
  }

  void _controllerListener() {
    if (isCurrent()) {
      if (!isResumed()) {
        onResume();
      }
    } else {
      if (isResumed()) {
        onPause();
      }
    }
  }

  TabController getTabController() {
    TabBarView tabBarView = _getTabBarView();
    if (tabBarView.controller != null) {
      return tabBarView.controller;
    } else {
      return DefaultTabController.of(context);
    }
  }

  TabBarView _getTabBarView() {
    TabBarView tabBarView = context.findAncestorWidgetOfExactType<TabBarView>();
    return tabBarView;
  }

  List<Widget> _getTabs() {
    return _getTabBarView().children;
  }

  bool isCurrent() {
    var currentWidget = _getTabs()[getTabController().index];
    /*if (currentWidget
        is extended.NestedScrollViewInnerScrollPositionKeyWidget) {
      currentWidget = (currentWidget as extended.NestedScrollViewInnerScrollPositionKeyWidget).child;
    }*/
    return widget == currentWidget && isParentCurrent();
  }

  bool isParentCurrent() {
    BasePageViewItemState pstate =
        context.findAncestorStateOfType<BasePageViewItemState>();
    BaseTabBarViewItemState tstate =
        context.findAncestorStateOfType<BaseTabBarViewItemState>();
    if (pstate != null) {
      return pstate.isCurrent();
    }
    if (tstate != null) {
      return tstate.isCurrent();
    }

    return true;
  }

  @override
  void onPageShow() {
    if (wantKeepAlive && isCurrent() && !isResumed()) {
      super.onPageShow();
    }
  }

  @override
  void onPageHide() {
    if (wantKeepAlive && isCurrent() && isResumed()) {
      super.onPageHide();
    }
  }

  @override
  void onBackground() {
    if (wantKeepAlive && isCurrent()) {
      super.onBackground();
    }
  }

  @override
  void onForeground() {
    if (wantKeepAlive && isCurrent()) {
      super.onForeground();
    }
  }
}
