import 'package:flutter/material.dart';
import 'package:extended_nested_scroll_view/extended_nested_scroll_view.dart' as extended;
import 'package:flutter/gestures.dart' show DragStartBehavior;

typedef NestedScrollViewHeaderSliversBuilder = List<Widget> Function(
    BuildContext context, bool innerBoxIsScrolled);
typedef NestedScrollViewPinnedHeaderSliverHeightBuilder = double Function();

class INestedScrollView extends StatelessWidget {
  final ScrollController controller;
  final TabController tabController;
  final Axis scrollDirection;
  final bool reverse;
  final ScrollPhysics physics;
  final NestedScrollViewHeaderSliversBuilder headerSliverBuilder;
  final NestedScrollViewPinnedHeaderSliverHeightBuilder
      pinnedHeaderSliverHeightBuilder;
  final Widget body;
  final DragStartBehavior dragStartBehavior;
  final bool floatHeaderSlivers;
  final Clip clipBehavior;
  final String restorationId;

  INestedScrollView(
      {Key key,
      this.controller,
      this.tabController,
      this.scrollDirection = Axis.vertical,
      this.reverse = false,
      this.physics,
      this.pinnedHeaderSliverHeightBuilder,
      @required this.headerSliverBuilder,
      @required this.body,
      this.dragStartBehavior = DragStartBehavior.start,
      this.floatHeaderSlivers = false,
      this.clipBehavior = Clip.hardEdge,
      this.restorationId})
      : super(key: key);
  @override
  Widget build(BuildContext context) {
    return extended.ExtendedNestedScrollView(
      controller: controller,
      scrollDirection: scrollDirection,
      reverse: reverse,
      physics: physics,
      pinnedHeaderSliverHeightBuilder: pinnedHeaderSliverHeightBuilder,
      dragStartBehavior: dragStartBehavior,
      floatHeaderSlivers: floatHeaderSlivers,
      clipBehavior: clipBehavior,
      restorationId: restorationId,
      headerSliverBuilder: headerSliverBuilder,
      body: body,
    );
  }
}

class INestedTabBarView extends StatelessWidget {
  final Key key;
  final TabController controller;
  final ScrollPhysics physics;
  final DragStartBehavior dragStartBehavior;
  final List<Widget> children;
  INestedTabBarView({
    this.key,
    @required this.children,
    this.controller,
    this.physics,
    this.dragStartBehavior = DragStartBehavior.start,
  });
  @override
  Widget build(BuildContext context) {
    return TabBarView(
          key: key,
          controller: controller,
          physics: physics,
          dragStartBehavior: dragStartBehavior,
          children: List.generate(children.length, (index) {
            return children[index];
          }),
        );
  }
}
