import 'dart:io';

import 'package:flutter_cache_manager/flutter_cache_manager.dart';

///
/// 文件缓存工具
///
class FileCacheUtils {
  static FileCacheUtils _instance;
  static FileCacheUtils getInstance() {
    if (_instance == null) {
      _instance = new FileCacheUtils();
    }
    return _instance;
  }

  ///
  /// 下载文件
  /// [filePath]              待下载的文件路径
  ///
  Future<FileEntity> downloadFile({String filePath}) async {
    FileInfo result = await DefaultCacheManager().downloadFile(filePath);
    return FileEntity(
      file: result.file,
      source: result.source != null ? result.source.index : -1,
      validTill: result.validTill,
      originalUrl: result.originalUrl,
    );
  }
}

class FileEntity {
  FileEntity({this.file, this.source, this.validTill, this.originalUrl});
  final String originalUrl;

  /// Fetched file
  final File file;

  /// Source from the file, can be cache or online (web).
  final int source;

  /// Validity date of the file. After this date the validity is not guaranteed
  /// and the CacheManager will try to update the file.
  final DateTime validTill;
}
