import 'package:mvp/mvp.dart';
import 'package:flutter/material.dart';
import 'package:flutter/cupertino.dart';
import 'package:common_module/components/i_empty_status.dart';

class ListModelStatusUtils<T extends BaseListModel> {
  static ListModelStatusUtils _instance;

  static ListModelStatusUtils getInstance() {
    if (_instance == null) {
      _instance = new ListModelStatusUtils._();
    }
    return _instance;
  }

  ListModelStatusUtils._();

  Widget form(
      {T model,
      double paddingTop = 139,
      double loadingPaddingTop = 139,
      String failText = '暂无数据',
      String btnText = "点击刷新",
      Color btnColor,
      double btnRadius,
      String icon,
      double iconSize,
      Function() onTap}) {
    // 未请求完成 && 在第一页 显示加载中
    if (!model.isRequestDone() && model.getPage() == 1) {
      return Container(
        padding: EdgeInsets.only(top: loadingPaddingTop),
        child: CupertinoActivityIndicator(
          radius: 12,
        ),
      );
    }

    // 请求完成  && 加载的是第一页的时候  && 列表数据长度为0时,显示空组件
    if (model.isRequestDone() && model.getList().length == 0) {
      return IEmptyStatus(
        icon: icon,
        iconSize: iconSize,
        btnText: btnText,
        btnColor: btnColor,
        btnRadius: btnRadius,
        paddingTop: paddingTop,
        msg: !model.isLoadFail() ? failText : '加载失败,请刷新重试',
        onTap: (netWorkConnectd) {
          if (onTap != null) {
            onTap();
          } else {
            model.refresh();
          }
        },
      );
    }
    return null;
  }
}
