import 'package:common_module/utils/permission_utils.dart';
import 'package:permission_handler/permission_handler.dart';

///
/// 定位权限相关类
///
class LocationPermissionUtils {
  static LocationPermissionUtils _instance;
  static LocationPermissionUtils getInstance() {
    if (_instance == null) {
      _instance = new LocationPermissionUtils();
    }
    return _instance;
  }

  ///
  /// 判断是否有定位权限
  ///
  Future<bool> hasPermission() async {
    Map<Permission, PermissionUtilsStatus> result =
        await PermissionUtils.getInstance()
            .has(permissions: [Permission.location]);
    return result[Permission.location] == PermissionUtilsStatus.granted;
  }

  ///
  /// 请求定位权限
  ///
  Future<bool> reqPermission() async {
    Map<Permission, PermissionUtilsStatus> result =
        await PermissionUtils.getInstance()
            .has(permissions: [Permission.location]);
    var location = result[Permission.location];
    if (location == PermissionUtilsStatus.permanentlyDenied) {
      PermissionUtils.getInstance().openAppSetting();
      return false;
    } else if (location == PermissionUtilsStatus.denied) {
      result = await PermissionUtils.getInstance()
          .request(permissions: [Permission.location]);
      location = result[Permission.location];

      return location == PermissionUtilsStatus.granted;
    }else if (location == PermissionUtilsStatus.granted) {
      return true;
    }
    return false;
  }
}
