import 'package:common_module/eventbus/nav_tabbar_event.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter_boost/flutter_boost.dart';
import 'package:url_launcher/url_launcher.dart';

import 'event_bus_utils.dart';

/// 路由前置拦截器
typedef PopBeforeFuntion = Future<void> Function(
    String path, bool isNative, bool needLogin, Map<String, dynamic> arguments);

///
/// Boost导航路由管理器
///
class NavigateUtils {
  /// 路由前置拦截器
  static PopBeforeFuntion _onPopBefore;

  ///
  /// 开启新页面统一API
  /// [path]              待打开页面的路径
  /// [isNative]          是否需伴随原生容器弹出，默认false
  /// [needLogin]         是否需要登录
  /// [arguments]         携带到下一页面的参数
  ///
  ///
  static Future<Map<dynamic, dynamic>> push({
    @required String path,
    bool isNative = false,
    bool needLogin = true,
    Map<String, dynamic> arguments,
  }) async {
    if (_onPopBefore != null) {
      await _onPopBefore(path, isNative, needLogin, arguments);
    }

    if (arguments == null) {
      arguments = {};
    }
    // 记录上一个页面的名称
    arguments['previousRouteName'] = _getPreviousRouteName();
    dynamic result = await BoostNavigator.instance.push(
      path,
      withContainer: isNative,
      arguments: arguments,
    );
    return result;
  }

  ///
  ///
  ///
  static Future<bool> pushLaunch({
    @required String urlString,
    bool needLogin = true,
  }) async {
    if (_onPopBefore != null) {
      await _onPopBefore(urlString, false, needLogin, {});
    }
    var urlencode = Uri.encodeComponent(urlString);
    return await launch(
        "imeituan://www.meituan.com/web?lch=appshare_ker3284penn1&wkwebview=1&noquery=1&notitlebar=0&url=$urlencode");
  }

  ///
  /// 关闭当前页面
  /// [arguments]               关闭页面时携带的参数
  ///
  static Future<bool> pop({Map<String, dynamic> arguments = const {}}) async {
    dynamic result = BoostNavigator.instance.pop(arguments);
    return result;
  }
  
  ///
  ///
  /// 返回到指定导航页面
  /// [index]           第index个索引
  /// [arguments]       返回参数
  ///
  static Future<bool> popTabBarNavigation(String key,{Map<String, dynamic> arguments = const {}}) async{
    eventBus.fire(NavTabBarEvent(key: key,tabEnum:NavTabBarEnum.TabBarPageChange,arguments:arguments));
    Future.delayed(Duration(milliseconds: 50), () async{
      await NavigateUtils.popRoot(rootPath: '/main');
    });
    return Future.value(true);
  }

  ///
  /// 回到主页
  /// [rootPath]            根路由路径
  ///
  static popRoot({
    @required String rootPath,
    Map<String, dynamic> arguments = const {}
  }) async {
    PageInfo pageInfo = BoostNavigator.instance.getTopPageInfo();
    String pageName = pageInfo?.pageName ?? '';
    if(pageName != rootPath){
      await BoostNavigator.instance.pop().then((value) async {
        PageInfo pageInfo = BoostNavigator.instance.getTopPageInfo();
        String pageName = pageInfo?.pageName ?? '';
        if (pageName.isNotEmpty && pageName != rootPath) {
          await popRoot(rootPath: rootPath,arguments:arguments);
        }
      }).whenComplete(() async {});
    }
  }

  /// 设置路由前置拦截器
  static void setPopBefore(PopBeforeFuntion popBeforeFuntion) {
    _onPopBefore = popBeforeFuntion;
  }

  // 获取下一个新页面的上一个页面名称(页面路径)
  static String _getPreviousRouteName() {
    PageInfo pageInfo = BoostNavigator.instance.getTopPageInfo();
    var boostPath = pageInfo?.pageName;
    return boostPath;
  }
}
