import 'dart:io';
import 'package:flutter/foundation.dart';
import 'package:localstorage/localstorage.dart';
//import 'package:shared_preferences/shared_preferences.dart';
import 'package:path_provider/path_provider.dart';

///
/// 数据持久化工具
///
class StoreUtils {
  /// app全局配置 eg:theme
  //SharedPreferences _sharedPreferences;

  /// 临时目录 eg: cookie
  Directory _temporaryDirectory;

  /// 初始化必备操作 eg:user数据
  LocalStorage _localStorage;

  static StoreUtils _instance;
  static StoreUtils getInstance() {
    if (_instance == null) {
      _instance = new StoreUtils();
    }
    return _instance;
  }

  ///
  /// 初始化
  ///
  Future<bool> init() async {
    // async 异步操作
    // sync 同步操作
    //_sharedPreferences = await SharedPreferences.getInstance();
    _localStorage = LocalStorage('LocalStorage');
    _temporaryDirectory = await getTemporaryDirectory();
    return await _localStorage.ready;
  }

  ///
  /// 根据[key]获取相应的值
  /// [key]                       键
  ///
  dynamic getForKey({@required String key,dynamic defaultValue}) {
    return getStorage.getItem(key)??defaultValue;
  }

  ///
  /// 设置[key]以及对应的[value]
  /// [key]                         键
  /// [value]                       值
  ///
  Future<void> setKV({@required String key, @required dynamic value}) {
    return getStorage?.setItem(key, value);
  }

  ///
  /// 根据[key]删除对应的值
  /// [key]                         键
  ///
  Future<void> delForkey({@required String key}) {
    return getStorage.deleteItem(key);
  }

  ///
  /// 获取本地存储类
  ///
  LocalStorage get getStorage {
    return _localStorage;
  }

  ///
  /// 获取数据持久化的类
  ///
  /*SharedPreferences get getShared {
    return _sharedPreferences;
  }*/

  ///
  /// 获取缓存的文件夹
  ///
  Directory get getTempDir {
    return _temporaryDirectory;
  }
}
