import 'package:flutter/material.dart';
import 'package:common_module/widget/image/index.dart';

var isAliCDN = RegExp('alicdn.com');
var isCache = RegExp(r'_\d+x\d+((Q\d+s\d+)?).(jpg|png|gif|jpeg)');

///公用的图片控件
///[imageUrl]图片地址
///[progressImage]占位图片名
///[fit]图片适应方式[BoxFit]
///[width]图片宽度
///[height]图片高度
///[placeholderFit]占位适应方式[BoxFit]
class XiaoxiongBaseImageWidget extends StatelessWidget {
  final String imageUrl;
  final String heroTag;
  final String progressImage;
  final BoxFit fit;
  final double width;
  final double height;
  final BoxFit placeholderFit;
  final AlignmentGeometry alignment;
  final Widget Function(BuildContext, String) placeholder;

  String get _imageUrl {
    // var isAli = isAliCDN.hasMatch(imageUrl);
    // var isCnd = !isCache.hasMatch(imageUrl);

    // if (isAli && isCnd) {
    //   return "${imageUrl}_400x400Q50s50.jpg";
    // }
    return imageUrl;
  }

  const XiaoxiongBaseImageWidget(
      {Key key,
      @required this.imageUrl,
      this.progressImage,
      this.fit = BoxFit.cover,
      this.width,
      this.height,
      this.placeholderFit = BoxFit.contain,
      this.placeholder,
      this.heroTag,
      this.alignment = Alignment.center})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    if (heroTag != null && heroTag.isNotEmpty) {
      return Hero(tag: heroTag, child: buildCachedNetworkImage());
    }
    return Container(
      child: buildCachedNetworkImage(),
    );
  }

  Widget buildCachedNetworkImage() {
    if (imageUrl == null || imageUrl.isEmpty) {
      return Container(
        width: width,
        height: height,
      );
    } else if (!imageUrl.startsWith('http')) {
      return Image.asset(
        imageUrl,
        fit: fit,
        width: width,
        height: height,
      );
    }
    return CachedNetworkImage(
      alignment: alignment,
      imageUrl: _imageUrl ?? '',
      fit: fit,
      width: width,
      height: height,
      placeholder: placeholder ??
          (context, index) => Container(
                width: width,
                height: height,
                color: Color.fromARGB(255, 233, 233, 233),
                child: Image.asset(
                  progressImage ?? "assets/placeholderImage.png",
                  fit: placeholderFit,
                ),
              ),
    );
  }
}
